@@ -48,11 +48,11 @@
 #include <stdint.h>
 
 #if !defined(__APPLE__) && !defined(__FreeBSD__)
-#   include "config.h"
+#  include "config.h"
 #endif
 
 #if defined(__FreeBSD__)
-#   include <netinet/in.h>
+#  include <netinet/in.h>
 #endif
 
 #ifndef _WIN32
@@ -66,10 +66,12 @@
 #include <poll.h>
 #include <netdb.h>
 #else
-# include "include/sys/event.h"
-# include "src/windows/platform.h"
+#  include "include/sys/event.h"
+#  include "src/windows/platform.h"
 #endif
 
+#include "config.h"
+
 /** Convenience macros
  *
  */
@@ -78,14 +80,19 @@
 struct test_context;
 
 struct unit_test {
-    const char *ut_name;
-    int         ut_enabled;
-    void      (*ut_func)(struct test_context *);
+    const char   *ut_name;
+    int          ut_enabled;
+    void         (*ut_func)(struct test_context *);
+    int          ut_start;
+    int          ut_end;
+    unsigned int ut_num;
 };
 
 #define MAX_TESTS 50
 struct test_context {
     struct unit_test tests[MAX_TESTS];
+    struct unit_test *test; /* Current test being run */
+
     char *cur_test_id;
     int iterations;
     int iteration;
@@ -110,20 +117,26 @@ void test_evfilt_proc(struct test_context *);
 #ifdef EVFILT_USER
 void test_evfilt_user(struct test_context *);
 #endif
+void test_evfilt_libkqueue(struct test_context *);
 
-#define test(f,ctx,...) do {                                            \
-    assert(ctx != NULL); \
-    test_begin(ctx, "test_"#f"()\t"__VA_ARGS__); \
-    errno = 0; \
-    test_##f(ctx); \
-    test_end(ctx); \
+#define test(f, ctx ,...) do {                                            \
+    if ((ctx->test->ut_num >= ctx->test->ut_start) && (ctx->test->ut_num <= ctx->test->ut_end)) {\
+        assert(ctx != NULL); \
+        test_begin(ctx, "test_"#f"()\t"__VA_ARGS__); \
+        errno = 0; \
+        test_##f(ctx); \
+        test_end(ctx); \
+    } \
+    ctx->test->ut_num++; \
 } while (/*CONSTCOND*/0)
 
 extern const char * kevent_to_str(struct kevent *);
-void kevent_get(struct kevent *, int, int);
-void kevent_get_hires(struct kevent *, int, struct timespec *);
-int kevent_get_timeout(struct kevent *, int, struct timespec *);
+void kevent_get(struct kevent kev[], int numevents, int kqfd, int expect_rv);
+void kevent_get_hires(struct kevent kev[], int numevents, int kqfd, struct timespec *timeout);
+int kevent_get_timeout(struct kevent kev[], int numevents, int kqfd, struct timespec *timeout);
+
 void kevent_update(int kqfd, struct kevent *kev);
+void kevent_update_expect_fail(int kqfd, struct kevent *kev);
 
 #define kevent_cmp(a,b) _kevent_cmp(a,b, __FILE__, __LINE__)
 void _kevent_cmp(struct kevent *expected, struct kevent *got, const char *file, int line);
@@ -140,8 +153,21 @@ kevent_add(int kqfd, struct kevent *kev,
         intptr_t  data,
         void      *udata);
 
+
+#define kevent_add_with_receipt(...) _kevent_add_with_receipt(__VA_ARGS__, __FILE__, __LINE__)
+void
+_kevent_add_with_receipt(int kqfd, struct kevent *kev,
+        uintptr_t ident,
+        short     filter,
+        u_short   flags,
+        u_int     fflags,
+        intptr_t  data,
+        void      *udata,
+        char const *file,
+        int line);
+
 /* Checks if any events are pending, which is an error. */
-#define test_no_kevents(a) _test_no_kevents(a, __FILE__, __LINE__)
+#define test_no_kevents(_kq) _test_no_kevents(_kq, __FILE__, __LINE__)
 void _test_no_kevents(int, const char *, int);
 
 unsigned int print_fd_table(void);
