@@ -382,9 +382,7 @@ TEST_CASE("advance") {
 
         // Match on `A`.
         {
-            int rc;
-            hilti::rt::stream::View ncur;
-            std::tie(rc, ncur) = re.tokenMatcher().advance(cur);
+            auto [rc, ncur] = re.tokenMatcher().advance(cur);
             CHECK_EQ(rc, 1);
             CHECK_EQ(ncur, stream::View(cur.begin() + 1, cur.end()));
             cur = ncur;
@@ -401,9 +399,7 @@ TEST_CASE("advance") {
 
         // Match on `B`.
         {
-            int rc;
-            hilti::rt::stream::View ncur;
-            std::tie(rc, ncur) = re.tokenMatcher().advance(cur);
+            auto [rc, ncur] = re.tokenMatcher().advance(cur);
             CHECK_EQ(rc, 1);
             CHECK_EQ(ncur.offset(), 1 + 1024 + 1);
         }
